"""
A module for an angle class

The angle class is another "number-like" class, which means
we need to use the Python data model to provide all of the
different operators.

Author: Walker M. White
Data: November 2, 2020
"""
import math


class Angle(object):
    """
    A class representing an angle in DMS
    (degrees, minutes, seconds) format

    The class does not allow a finer grained measurement
    than seconds (e.g. microseconds). All of the values
    degrees, minutes, and seconds must be integral.
    """
    # Attribute _degrees: The angle in degrees
    # Invariant: _degrees is (any) int
    #
    # Attribute _minutes: Part of single degree
    # Invariant: _minutes is an int 0..59
    #
    # Attribute _seconds: Part of single minute
    # Invariant: _seconds is an int 0..59
    #


    # We will give the attributes to get you started
    def getDegrees(self):
        """
        Returns the number of degrees
        """
        return self._degrees

    def setDegrees(self,value):
        """
        Sets the number of degrees
        
        Paramater value: The number of degrees
        Precondition: value is an int
        """
        assert type(value) == int
        self._degrees = value

    def getMinutes(self):
        """
        Returns the number of minutes
        """
        return self._minutes

    def setMinutes(self,value):
        """
        Sets the number of minutes
        
        Paramater value: The number of minutes
        Precondition: value is an int 0..59
        """
        assert type(value) == int
        assert value >= 0 and value < 60
        self._minutes = value

    def getSeconds(self):
        """
        Returns the number of seconds
        """
        return self._seconds

    def setSeconds(self,value):
        """
        Sets the number of seconds
        
        Paramater value: The number of seconds
        Precondition: value is an int 0..59
        """
        assert type(value) == int
        assert value >= 0 and value < 60
        self._seconds = value


    def __init__(self,d,m=0,s=0):
        """
        Initializes a new angle
        
        Parameter d: The number of degrees
        Precondition: d is an int
        
        Parameter m: The number of minutes (OPTIONAL; default 0)
        Precondition: m is an int 0..59
        
        Parameter s: The number of seconds (OPTIONAL; default 0)
        Precondition: s is an int 0..59
        """
        pass

    def __str__(self):
        """
        Returns an informal string representation of this angle
        
        The string has the form d° m' s"
        """
        pass

    def __repr__(self):
        """
        Returns a formal string representation of this angle

        The string has the form Angle[d° m' s"]
        """
        return str(self.__class__)+'['+str(self)+']'

    def __eq__(self,value):
        """
        Returns true of value is an Angle equals to this one

        Parameter value: The object to compare
        Precondition: ?????
        """
        pass

    def __add__(self,value):
        """
        Returns an angle that the sum of this angle and value
        
        Parameter value: The angle to add
        Precondition: value is an Angle
        """
        pass

    def __sub__(self,value):
        """
        Returns an angle that the difference of this angle and value
        
        Parameter value: The angle to subtract
        Precondition: value is an Angle
        """
        pass

    def __mul__(self,value):
        """
        Returns an angle that the multiple of this angle and value
        
        Parameter value: The value to multiply by
        Precondition: value is an int
        """
        pass

    def __floordiv__(self,value):
        """
        Returns an angle that the result of dividing this angle by value
        
        Parameter value: The angle to subtract
        Precondition: value is an int
        """
        pass

    @classmethod
    def forDegrees(cls,d):
        """
        Returns the angle for the given degrees as a decimal
        
        Because seconds must be an int, there is no guarantee that we
        can make an angle equal to d. The angle will be the floor,
        which is the largest angle less than or equal to d
        
        Parameter d: The decimal degrees
        Precondition: d is a float
        """
        pass

    @classmethod
    def forRadians(cls,r):
        """
        Returns the angle for the given radians
        
        Because seconds must be an int, there is no guarantee that we
        can make an angle equal to r. The angle will be the floor,
        which is the largest angle less than or equal to r
        
        Parameter r: The angle radians
        Precondition: r is a float
        """
        pass
